@@ -26,6 +26,7 @@ from typing import (
     TypeVar,
     Union,
     cast,
+    overload,
 )
 from weakref import WeakSet, WeakValueDictionary
 
@@ -37,7 +38,7 @@ from rich.protocol import is_renderable
 from rich.segment import Segment, Segments
 from rich.traceback import Traceback
 
-from . import Logger, LogGroup, LogVerbosity, actions, events, log, messages
+from . import actions, Logger, LogGroup, LogVerbosity, events, log, messages
 from ._animator import DEFAULT_EASING, Animatable, Animator, EasingFunction
 from ._ansi_sequences import SYNC_END, SYNC_START
 from ._callback import invoke
@@ -46,6 +47,7 @@ from ._event_broker import NoHandler, extract_handler_actions
 from ._filter import LineFilter, Monochrome
 from ._path import _make_path_object_relative
 from ._typing import Final, TypeAlias
+from .actions import SkipAction
 from .await_remove import AwaitRemove
 from .binding import Binding, Bindings
 from .css.query import NoMatches
@@ -62,7 +64,7 @@ from .messages import CallbackType
 from .reactive import Reactive
 from .renderables.blank import Blank
 from .screen import Screen
-from .widget import AwaitMount, MountError, Widget
+from .widget import AwaitMount, Widget
 
 
 if TYPE_CHECKING:
@@ -236,6 +238,12 @@ class App(Generic[ReturnType], DOMNode):
     TITLE: str | None = None
     SUB_TITLE: str | None = None
 
+    BINDINGS = [
+        Binding("ctrl+c", "quit", "Quit", show=False, priority=True),
+        Binding("tab", "focus_next", "Focus Next", show=False),
+        Binding("shift+tab", "focus_previous", "Focus Previous", show=False),
+    ]
+
     title: Reactive[str] = Reactive("")
     sub_title: Reactive[str] = Reactive("")
     dark: Reactive[bool] = Reactive(True)
@@ -297,7 +305,6 @@ class App(Generic[ReturnType], DOMNode):
 
         self._logger = Logger(self._log)
 
-        self._bindings.bind("ctrl+c", "quit", show=False, universal=True)
         self._refresh_required = False
 
         self.design = DEFAULT_COLORS
@@ -347,6 +354,7 @@ class App(Generic[ReturnType], DOMNode):
                 self.devtools = DevtoolsClient()
 
         self._return_value: ReturnType | None = None
+        self._exit = False
 
         self.css_monitor = (
             FileMonitor(self.css_path, self._on_css_change)
@@ -355,6 +363,8 @@ class App(Generic[ReturnType], DOMNode):
         )
         self._screenshot: str | None = None
         self._dom_lock = asyncio.Lock()
+        self._dom_ready = False
+        self.set_class(self.dark, "-dark-mode")
 
     @property
     def return_value(self) -> ReturnType | None:
@@ -412,14 +422,20 @@ class App(Generic[ReturnType], DOMNode):
         """list[Screen]: A *copy* of the screen stack."""
         return self._screen_stack.copy()
 
-    def exit(self, result: ReturnType | None = None) -> None:
+    def exit(
+        self, result: ReturnType | None = None, message: RenderableType | None = None
+    ) -> None:
         """Exit the app, and return the supplied result.
 
         Args:
             result (ReturnType | None, optional): Return value. Defaults to None.
+            message (RenderableType | None): Optional message to display on exit.
         """
+        self._exit = True
         self._return_value = result
         self.post_message_no_wait(messages.ExitApp(sender=self))
+        if message:
+            self._exit_renderables.append(message)
 
     @property
     def focused(self) -> Widget | None:
@@ -460,7 +476,14 @@ class App(Generic[ReturnType], DOMNode):
         """Watches the dark bool."""
         self.set_class(dark, "-dark-mode")
         self.set_class(not dark, "-light-mode")
-        self.refresh_css()
+        try:
+            self.refresh_css()
+        except ScreenStackError:
+            # It's possible that `dark` can be set before we have a default
+            # screen, in an app's `on_load`, for example. So let's eat the
+            # ScreenStackError -- the above styles will be handled once the
+            # screen is spun up anyway.
+            pass
 
     def get_driver_class(self) -> Type[Driver]:
         """Get a driver class for this platform.
@@ -892,23 +915,52 @@ class App(Generic[ReturnType], DOMNode):
     def render(self) -> RenderableType:
         return Blank(self.styles.background)
 
+    ExpectType = TypeVar("ExpectType", bound=Widget)
+
+    @overload
     def get_child_by_id(self, id: str) -> Widget:
+        ...
+
+    @overload
+    def get_child_by_id(self, id: str, expect_type: type[ExpectType]) -> ExpectType:
+        ...
+
+    def get_child_by_id(
+        self, id: str, expect_type: type[ExpectType] | None = None
+    ) -> ExpectType | Widget:
         """Shorthand for self.screen.get_child(id: str)
         Returns the first child (immediate descendent) of this DOMNode
         with the given ID.
 
         Args:
             id (str): The ID of the node to search for.
+            expect_type (type | None, optional): Require the object be of the supplied type, or None for any type.
+                Defaults to None.
 
         Returns:
-            DOMNode: The first child of this node with the specified ID.
+            ExpectType | Widget: The first child of this node with the specified ID.
 
         Raises:
             NoMatches: if no children could be found for this ID
+            WrongType: if the wrong type was found.
         """
-        return self.screen.get_child_by_id(id)
+        return (
+            self.screen.get_child_by_id(id)
+            if expect_type is None
+            else self.screen.get_child_by_id(id, expect_type)
+        )
 
+    @overload
     def get_widget_by_id(self, id: str) -> Widget:
+        ...
+
+    @overload
+    def get_widget_by_id(self, id: str, expect_type: type[ExpectType]) -> ExpectType:
+        ...
+
+    def get_widget_by_id(
+        self, id: str, expect_type: type[ExpectType] | None = None
+    ) -> ExpectType | Widget:
         """Shorthand for self.screen.get_widget_by_id(id)
         Return the first descendant widget with the given ID.
 
@@ -918,14 +970,21 @@ class App(Generic[ReturnType], DOMNode):
 
         Args:
             id (str): The ID to search for in the subtree
+            expect_type (type | None, optional): Require the object be of the supplied type, or None for any type.
+                Defaults to None.
 
         Returns:
-            DOMNode: The first descendant encountered with this ID.
+            ExpectType | Widget: The first descendant encountered with this ID.
 
         Raises:
             NoMatches: if no children could be found for this ID
+            WrongType: if the wrong type was found.
         """
-        return self.screen.get_widget_by_id(id)
+        return (
+            self.screen.get_widget_by_id(id)
+            if expect_type is None
+            else self.screen.get_widget_by_id(id, expect_type)
+        )
 
     def update_styles(self, node: DOMNode | None = None) -> None:
         """Request update of styles.
@@ -1042,9 +1101,9 @@ class App(Generic[ReturnType], DOMNode):
         _screen = self.get_screen(screen)
         if not _screen.is_running:
             widgets = self._register(self, _screen)
-            return (_screen, AwaitMount(widgets))
+            return (_screen, AwaitMount(_screen, widgets))
         else:
-            return (_screen, AwaitMount([]))
+            return (_screen, AwaitMount(_screen, []))
 
     def _replace_screen(self, screen: Screen) -> Screen:
         """Handle the replaced screen.
@@ -1090,7 +1149,7 @@ class App(Generic[ReturnType], DOMNode):
             self.screen.post_message_no_wait(events.ScreenResume(self))
             self.log.system(f"{self.screen} is current (SWITCHED)")
             return await_mount
-        return AwaitMount([])
+        return AwaitMount(self.screen, [])
 
     def install_screen(self, screen: Screen, name: str | None = None) -> AwaitMount:
         """Install a screen.
@@ -1339,11 +1398,10 @@ class App(Generic[ReturnType], DOMNode):
                 raise
 
             finally:
+                self._running = True
                 await self._ready()
                 await invoke_ready_callback()
 
-            self._running = True
-
             try:
                 await self._process_messages_loop()
             except asyncio.CancelledError:
@@ -1368,28 +1426,29 @@ class App(Generic[ReturnType], DOMNode):
             )
             driver = self._driver = driver_class(self.console, self, size=terminal_size)
 
-            driver.start_application_mode()
-            try:
-                if headless:
-                    await run_process_messages()
-                else:
-                    if self.devtools is not None:
-                        devtools = self.devtools
-                        assert devtools is not None
-                        from .devtools.redirect_output import StdoutRedirector
-
-                        redirector = StdoutRedirector(devtools)
-                        with redirect_stderr(redirector):
-                            with redirect_stdout(redirector):  # type: ignore
-                                await run_process_messages()
+            if not self._exit:
+                driver.start_application_mode()
+                try:
+                    if headless:
+                        await run_process_messages()
                     else:
-                        null_file = _NullFile()
-                        with redirect_stderr(null_file):
-                            with redirect_stdout(null_file):
-                                await run_process_messages()
+                        if self.devtools is not None:
+                            devtools = self.devtools
+                            assert devtools is not None
+                            from .devtools.redirect_output import StdoutRedirector
 
-            finally:
-                driver.stop_application_mode()
+                            redirector = StdoutRedirector(devtools)
+                            with redirect_stderr(redirector):
+                                with redirect_stdout(redirector):  # type: ignore
+                                    await run_process_messages()
+                        else:
+                            null_file = _NullFile()
+                            with redirect_stderr(null_file):
+                                with redirect_stdout(null_file):
+                                    await run_process_messages()
+
+                finally:
+                    driver.stop_application_mode()
         except Exception as error:
             self._handle_exception(error)
 
@@ -1435,7 +1494,7 @@ class App(Generic[ReturnType], DOMNode):
             nodes: set[DOMNode] = {
                 child
                 for node in self._require_stylesheet_update
-                for child in node.walk_children()
+                for child in node.walk_children(with_self=True)
             }
             self._require_stylesheet_update.clear()
             self.stylesheet.update_nodes(nodes, animate=True)
@@ -1463,7 +1522,6 @@ class App(Generic[ReturnType], DOMNode):
 
         # If we don't already know about this widget...
         if child not in self._registry:
-
             # Now to figure out where to place it. If we've got a `before`...
             if before is not None:
                 # ...it's safe to NodeList._insert before that location.
@@ -1526,7 +1584,6 @@ class App(Generic[ReturnType], DOMNode):
                 if widget.children:
                     self._register(widget, *widget.children)
                 apply_stylesheet(widget)
-
         return list(widgets)
 
     def _unregister(self, widget: Widget) -> None:
@@ -1692,22 +1749,23 @@ class App(Generic[ReturnType], DOMNode):
             ]
         return namespace_bindings
 
-    async def check_bindings(self, key: str, universal: bool = False) -> bool:
+    async def check_bindings(self, key: str, priority: bool = False) -> bool:
         """Handle a key press.
 
         Args:
-            key (str): A key
-            universal (bool): Check universal keys if True, otherwise non-universal keys.
+            key (str): A key.
+            priority (bool): If `True` check from `App` down, otherwise from focused up.
 
         Returns:
             bool: True if the key was handled by a binding, otherwise False
         """
-
-        for namespace, bindings in self._binding_chain:
+        for namespace, bindings in (
+            reversed(self._binding_chain) if priority else self._binding_chain
+        ):
             binding = bindings.keys.get(key)
-            if binding is not None and binding.universal == universal:
-                await self.action(binding.action, default_namespace=namespace)
-                return True
+            if binding is not None and binding.priority == priority:
+                if await self.action(binding.action, namespace):
+                    return True
         return False
 
     async def on_event(self, event: events.Event) -> None:
@@ -1725,7 +1783,7 @@ class App(Generic[ReturnType], DOMNode):
                 self.mouse_position = Offset(event.x, event.y)
                 await self.screen._forward_event(event)
             elif isinstance(event, events.Key):
-                if not await self.check_bindings(event.key, universal=True):
+                if not await self.check_bindings(event.key, priority=True):
                     forward_target = self.focused or self.screen
                     await forward_target._forward_event(event)
             else:
@@ -1776,32 +1834,41 @@ class App(Generic[ReturnType], DOMNode):
     async def _dispatch_action(
         self, namespace: object, action_name: str, params: Any
     ) -> bool:
+        """Dispatch an action to an action method.
+
+        Args:
+            namespace (object): Namespace (object) of action.
+            action_name (str): Name of the action.
+            params (Any): Action parameters.
+
+        Returns:
+            bool: True if handled, otherwise False.
+        """
+        _rich_traceback_guard = True
+
         log(
             "<action>",
             namespace=namespace,
             action_name=action_name,
             params=params,
         )
-        _rich_traceback_guard = True
 
-        public_method_name = f"action_{action_name}"
-        private_method_name = f"_{public_method_name}"
-
-        private_method = getattr(namespace, private_method_name, None)
-        public_method = getattr(namespace, public_method_name, None)
-
-        if private_method is None and public_method is None:
+        try:
+            private_method = getattr(namespace, f"_action_{action_name}", None)
+            if callable(private_method):
+                await invoke(private_method, *params)
+                return True
+            public_method = getattr(namespace, f"action_{action_name}", None)
+            if callable(public_method):
+                await invoke(public_method, *params)
+                return True
             log(
-                f"<action> {action_name!r} has no target. Couldn't find methods {public_method_name!r} or {private_method_name!r}"
+                f"<action> {action_name!r} has no target."
+                f" Could not find methods '_action_{action_name}' or 'action_{action_name}'"
             )
-
-        if callable(private_method):
-            await invoke(private_method, *params)
-            return True
-        elif callable(public_method):
-            await invoke(public_method, *params)
-            return True
-
+        except SkipAction:
+            # The action method raised this to explicitly not handle the action
+            log("<action> {action_name!r} skipped.")
         return False
 
     async def _broker_event(
@@ -1812,7 +1879,7 @@ class App(Generic[ReturnType], DOMNode):
         Args:
             event_name (str): _description_
             event (events.Event): An event object.
-            default_namespace (object | None): TODO: _description_
+            default_namespace (object | None): The default namespace, where one isn't supplied.
 
         Returns:
             bool: True if an action was processed.
@@ -1842,13 +1909,8 @@ class App(Generic[ReturnType], DOMNode):
         message.stop()
 
     async def _on_key(self, event: events.Key) -> None:
-        if event.key == "tab":
-            self.screen.focus_next()
-        elif event.key == "shift+tab":
-            self.screen.focus_previous()
-        else:
-            if not (await self.check_bindings(event.key)):
-                await self.dispatch_key(event)
+        if not (await self.check_bindings(event.key)):
+            await self.dispatch_key(event)
 
     async def _on_shutdown_request(self, event: events.ShutdownRequest) -> None:
         log("shutdown request")
@@ -1918,24 +1980,6 @@ class App(Generic[ReturnType], DOMNode):
         # prune event.
         return pruned_remove
 
-    async def _on_prune(self, event: events.Prune) -> None:
-        """Handle a prune event.
-
-        Args:
-            event (events.Prune): The prune event.
-        """
-
-        try:
-            # Prune all the widgets.
-            for widget in event.widgets:
-                await self._prune_node(widget)
-        finally:
-            # Finally, flag that we're done.
-            event.finished_flag.set()
-
-        # Flag that the layout needs refreshing.
-        self.refresh(layout=True)
-
     def _walk_children(self, root: Widget) -> Iterable[list[Widget]]:
         """Walk children depth first, generating widgets and a list of their siblings.
 
@@ -1977,9 +2021,9 @@ class App(Generic[ReturnType], DOMNode):
                 await self._prune_nodes(widgets)
             finally:
                 finished_event.set()
+                self.refresh(layout=True)
 
         removed_widgets = self._detach_from_dom(widgets)
-        self.refresh(layout=True)
 
         finished_event = asyncio.Event()
         asyncio.create_task(prune_widgets_task(removed_widgets, finished_event))
@@ -2023,7 +2067,8 @@ class App(Generic[ReturnType], DOMNode):
         self._unregister(root)
 
     async def action_check_bindings(self, key: str) -> None:
-        await self.check_bindings(key)
+        if not await self.check_bindings(key, priority=True):
+            await self.check_bindings(key, priority=False)
 
     async def action_quit(self) -> None:
         """Quit the app as soon as possible."""
@@ -2085,6 +2130,14 @@ class App(Generic[ReturnType], DOMNode):
     async def action_toggle_class(self, selector: str, class_name: str) -> None:
         self.screen.query(selector).toggle_class(class_name)
 
+    def action_focus_next(self) -> None:
+        """Focus the next widget."""
+        self.screen.focus_next()
+
+    def action_focus_previous(self) -> None:
+        """Focus the previous widget."""
+        self.screen.focus_previous()
+
     def _on_terminal_supports_synchronized_output(
         self, message: messages.TerminalSupportsSynchronizedOutput
     ) -> None:
