@@ -6,15 +6,11 @@ Functions and classes to manage terminal geometry (anything involving coordinate
 
 from __future__ import annotations
 
-import sys
 from functools import lru_cache
 from operator import attrgetter, itemgetter
 from typing import Any, Collection, NamedTuple, Tuple, TypeVar, Union, cast
 
-if sys.version_info >= (3, 10):
-    from typing import TypeAlias
-else:  # pragma: no cover
-    from typing_extensions import TypeAlias
+from textual._typing import TypeAlias
 
 SpacingDimensions: TypeAlias = Union[
     int, Tuple[int], Tuple[int, int], Tuple[int, int, int, int]
@@ -324,9 +320,9 @@ class Region(NamedTuple):
             Offset: An offset required to add to region to move it inside window_region.
         """
 
-        if region in window_region:
+        if region in window_region and not top:
             # Region is already inside the window, so no need to move it.
-            return Offset(0, 0)
+            return NULL_OFFSET
 
         window_left, window_top, window_right, window_bottom = window_region.corners
         region = region.crop_size(window_region.size)
@@ -345,19 +341,19 @@ class Region(NamedTuple):
                 key=abs,
             )
 
-        if not (
+        if top:
+            delta_y = top_ - window_top
+
+        elif not (
             (window_bottom > top_ >= window_top)
             and (window_bottom > bottom >= window_top)
         ):
             # The window needs to scroll on the Y axis to bring region in to view
-            if top:
-                delta_y = top_ - window_top
-            else:
-                delta_y = min(
-                    top_ - window_top,
-                    top_ - (window_bottom - region.height),
-                    key=abs,
-                )
+            delta_y = min(
+                top_ - window_top,
+                top_ - (window_bottom - region.height),
+                key=abs,
+            )
         return Offset(delta_x, delta_y)
 
     def __bool__(self) -> bool:
@@ -427,7 +423,7 @@ class Region(NamedTuple):
             Offset: Top left offset.
 
         """
-        return Offset(self.x, self.y)
+        return Offset(*self[:2])
 
     @property
     def bottom_left(self) -> Offset:
@@ -470,7 +466,7 @@ class Region(NamedTuple):
             Size: Size of the region.
 
         """
-        return Size(self.width, self.height)
+        return Size(*self[2:])
 
     @property
     def corners(self) -> tuple[int, int, int, int]:
@@ -639,6 +635,7 @@ class Region(NamedTuple):
             and (y2 >= oy2 >= y1)
         )
 
+    @lru_cache(maxsize=1024)
     def translate(self, offset: tuple[int, int]) -> Region:
         """Move the offset of the Region.
 
@@ -694,7 +691,8 @@ class Region(NamedTuple):
         Returns:
             Region: New region.
         """
-
+        if not any(margin):
+            return self
         top, right, bottom, left = margin
         x, y, width, height = self
         return Region(
